///
/// @file         update_imx6.h
/// @copyright    Copyright (c) 2015 Toradex AG
/// $Author: valter.minute $
/// $Revision: 0000 $
/// $Date: 2015-12-14 15:53:09 +0200 (Mon, 14 Dec 2015) $
/// @brief        Library to support updating bootloader and OS on Toradex modules.
/// @target       all Colibri and Apalis modules based on i.MX6.
///
///
/// @anchor imx6update_parameters
/// Update Library for Toradex modules. 
/// * Update flash memory sections from a file
/// * Backup flash memory sections to a file
///
/// @page pcfg_update Update Configuration Options
/// @section scfg_upd_imx IMX6 Update Configurations
/// @anchor  acfg_upd_imx
/// The following subset of Update configuration parameters is expected to be
/// available on any SoC.\n
/// Toradex  recommends to prefer these settings over the SoC specific
/// configurations. This can simplify the porting to any other Toradex hardware.
///
/// <table class="libParam" >
///   <tr><th>              Parameter Name</th><th>Value   </th><th> Description</th></tr>
///   <tr><td class="paramname"          >filename  </td><td>STRING  </td><td>Full path to the file to read write the backup to, or to read the update information from.</td></tr>
///   <tr><td class="paramname"          >format    </td><td>STRING  </td><td>The file format used. Some formats or sections may work only for read (R) or write (W)<br><ul><li>bootloader - raw(W)</li><li>os - raw(W),compressed(W)</li><li>configblock - raw(R/W)</li></ul></td></tr>
///   <tr><td class="paramname"          >sections  </td><td>STRING  </td><td>A comma-separated list of flash sections that will be backuped to the file, or restored from the file.<br>Currently on i.MX6 only a single section can be specified.<br>Supported sections on i.MX6 are<br><ul><li>bootloader</li><li>os</li><li>configblock</li></ul></td></tr>
///   <tr><td class="paramname"          >callback  </td><td>DWORD   </td><td>Optional pointer to a callback function.\n The callback function is used to report the progress of the update procedure and has to be a prototype defined as type @ref Upd_Callback.</td></tr>
/// </table>

#ifndef _UPDATE_IMX6_H_
#define _UPDATE_IMX6_H_

#ifdef __cplusplus
extern "C" {
#endif


//******************************************************************************
/// Initialize HANDLE for the Update Process.\n
/// @param[in]  registryPath    Registry path string value.e.g \c SOFTWARE\\Toradex\\Update.\n
///                             We placed this parameter here, even if we currently don't see any usage for registry settings for the Update Library.
/// @return                     Handle to the Update Process. NULL if the function failed.
HANDLE Imx6Upd_Init(const TCHAR *registryPath);

//******************************************************************************
/// De-initialize HANDLE for the Update Process.\n
/// @param[in]    hUpdate       HANDLE received from Imx6Upd_Init()
/// @retval       TRUE          Success
/// @retval       FALSE         Failure
BOOL Imx6Upd_Deinit(HANDLE hUpdate);

//******************************************************************************
/// Reads data from the module's flash storage, and stores it to a file.\n
/// The filename and sections need to be setup first through calls to Imx6Upd_SetConfigString().\n
/// Refer to the SoC specific documentation to learn details about the file format.
/// @param[in]  hUpdate         HANDLE received from Imx6Upd_Init()
/// @return     Number of Bytes backed up.
/// @retval     FALSE           Failure
DWORD Imx6Upd_Backup(HANDLE hUpdate);

//******************************************************************************
/// Read data from a file and restore it to the device's flash memory.\n
/// The filename and sections need to be setup first through calls to Imx6Upd_SetConfigString().\n
/// Refer to the SoC specific documentation to learn details about the file format.
/// @param[in]  hUpdate         HANDLE received from Imx6Upd_Init()
/// @return                     Number of Bytes restored
DWORD Imx6Upd_Restore(HANDLE hUpdate);

//******************************************************************************
/// Set a STRING configuration parameter.\n
/// NOTE: For security reasons, all previously configured parameters are
///       invalidated, if any error happens while executing this function!
/// @param[in]  hUpdate         HANDLE received from Imx6Upd_Init()
/// @param[in]  paramName       Parameter value to configure.\n
///                             Refer to @ref scfg_upd_imx for details.
/// @param[in]  value           String value to be set
/// @param[in]  storageType     StoreVolatile   = parameter will only be stored in local variable.
/// @retval     TRUE            Success
/// @retval     FALSE           Failure
BOOL Imx6Upd_SetConfigString(HANDLE hUpdate, const TCHAR *paramName, const TCHAR *value, ParamStorageType storageType);

//******************************************************************************
/// Set a DWORD configuration parameter.\n
/// NOTE: For security reasons, all previously configured parameters are
///       invalidated, if any error happens while executing this function!
/// @param[in]  hUpdate         HANDLE received from Imx6Upd_Init()
/// @param [in] paramName       Parameter value to configure.\n
///                             Refer to @ref scfg_upd_imx for details.
/// @param [in] value           Data to be set
/// @param[in]  storageType     StoreVolatile   = parameter will only be stored in local variable.
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Imx6Upd_SetConfigInt(HANDLE hUpdate, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Retrieves a STRING configuration parameter.
/// @param[in]   hUpdate        HANDLE received from Imx6Upd_Init()
/// @param[in]   paramName      Parameter value to retrieve.\n
///                             Refer to @ref scfg_upd_imx for details.
/// @param[out]  pValue         String data obtained
/// @param[in]   maxBytes       Maximum size of string in Bytes
/// @return      Number of bytes read. 0 indicates an error.
DWORD Imx6Upd_GetConfigString(HANDLE hUpdate, const TCHAR *paramName, TCHAR *pValue, DWORD maxBytes);

//******************************************************************************
/// Retrieves a DWORD configuration parameter.
/// @param[in]   hUpdate        HANDLE received from Imx6Upd_Init()
/// @param[in]   paramName      Parameter value to retrieve.\n
///                             Refer to @ref scfg_upd_imx for details.
/// @param[out]  pValue         DWORD data obtained
/// @return      Number of bytes read. 0 indicates an error.
DWORD Imx6Upd_GetConfigInt(HANDLE hUpdate, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Save the registry changes permanently to flash.\n
/// On some OS implementations it is possible to preselect saving the system part,
/// the user part or both parts from the registry (see @ref scfg_upd_imx)
/// @param[in]   hUpdate        HANDLE received from Imx6Upd_Init()
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Imx6Upd_RegistrySave(HANDLE hUpdate);

//******************************************************************************
/// Delete the flash registry information in the flash, i.e. restore the
/// factory-default registry settings.\n
/// On some OS implementations it is possible to preselect clearing the system part,
/// the user part or both parts from the registry (see @ref scfg_upd_imx)
/// @param[in]   hUpdate        HANDLE received from Imx6Upd_Init()
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Imx6Upd_RegistryClear(HANDLE hUpdate);

#ifdef __cplusplus
}
#endif

#endif //_UPDATE_IMX6_H_
